@@ -1,6 +1,6 @@
 ;;; admin.el --- utilities for Emacs administration
 
-;; Copyright (C) 2001-2014 Free Software Foundation, Inc.
+;; Copyright (C) 2001-2016 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -28,10 +28,6 @@
 
 (defvar add-log-time-format)		; in add-log
 
-;; Does this information need to be in every ChangeLog, as opposed to
-;; just the top-level one?  Only if you allow changes the same
-;; day as the release.
-;; http://lists.gnu.org/archive/html/emacs-devel/2013-03/msg00161.html
 (defun add-release-logs (root version &optional date)
   "Add \"Version VERSION released.\" change log entries in ROOT.
 Root must be the root of an Emacs source tree.
@@ -42,14 +38,12 @@ Optional argument DATE is the release date, default today."
 					  emacs-minor-version))
 		     (read-string "Release date: "
 				  (progn (require 'add-log)
-					 (let ((add-log-time-zone-rule t))
-					   (funcall add-log-time-format))))))
+                                         (funcall add-log-time-format nil t)))))
   (setq root (expand-file-name root))
   (unless (file-exists-p (expand-file-name "src/emacs.c" root))
     (user-error "%s doesn't seem to be the root of an Emacs source tree" root))
   (require 'add-log)
-  (or date (setq date (let ((add-log-time-zone-rule t))
-			(funcall add-log-time-format))))
+  (or date (setq date (funcall add-log-time-format nil t)))
   (let* ((logs (process-lines "find" root "-name" "ChangeLog"))
 	 (entry (format "%s  %s  <%s>\n\n\t* Version %s released.\n\n"
 			date
@@ -94,79 +88,82 @@ Root must be the root of an Emacs source tree."
 		       (rx (and "AC_INIT" (1+ (not (in ?,)))
                                 ?, (0+ space)
                                 (submatch (1+ (in "0-9."))))))
-  (set-version-in-file root "doc/emacs/emacsver.texi" version
-		       (rx (and "EMACSVER" (1+ space)
-				(submatch (1+ (in "0-9."))))))
-  (set-version-in-file root "doc/man/emacs.1" version
-		       (rx (and ".TH EMACS" (1+ not-newline)
-                                "GNU Emacs" (1+ space)
-				(submatch (1+ (in "0-9."))))))
-  (set-version-in-file root "nt/config.nt" version
-		       (rx (and bol "#" (0+ blank) "define" (1+ blank)
-				"VERSION" (1+ blank) "\""
-				(submatch (1+ (in "0-9."))))))
+  ;; TODO: msdos could easily extract the version number from
+  ;; configure.ac with sed, rather than duplicating the information.
   (set-version-in-file root "msdos/sed2v2.inp" version
 		       (rx (and bol "/^#undef " (1+ not-newline)
 				"define VERSION" (1+ space) "\""
 				(submatch (1+ (in "0-9."))))))
-  (set-version-in-file root "nt/makefile.w32-in" version
-		       (rx (and "VERSION" (0+ space) "=" (0+ space)
-				(submatch (1+ (in "0-9."))))))
-  ;; nt/emacs.rc also contains the version number, but in an awkward
-  ;; format. It must contain four components, separated by commas, and
-  ;; in two places those commas are followed by space, in two other
-  ;; places they are not.
-  (let* ((version-components (append (split-string version "\\.")
-				     '("0" "0")))
-	 (comma-version
-	  (concat (car version-components) ","
-		  (cadr version-components) ","
-		  (cadr (cdr version-components)) ","
-		  (cadr (cdr (cdr version-components)))))
-	 (comma-space-version
-	  (concat (car version-components) ", "
-		  (cadr version-components) ", "
-		  (cadr (cdr version-components)) ", "
-		  (cadr (cdr (cdr version-components))))))
-    (set-version-in-file root "nt/emacs.rc" comma-version
-			 (rx (and "FILEVERSION" (1+ space)
-				  (submatch (1+ (in "0-9,"))))))
-    (set-version-in-file root "nt/emacs.rc" comma-version
-			 (rx (and "PRODUCTVERSION" (1+ space)
-				  (submatch (1+ (in "0-9,"))))))
-    (set-version-in-file root "nt/emacs.rc" comma-space-version
-			 (rx (and "\"FileVersion\"" (0+ space) ?, (0+ space)
-				  ?\" (submatch (1+ (in "0-9, "))) "\\0\"")))
-    (set-version-in-file root "nt/emacs.rc" comma-space-version
-			 (rx (and "\"ProductVersion\"" (0+ space) ?,
-				  (0+ space) ?\" (submatch (1+ (in "0-9, ")))
-				  "\\0\"")))
-    ;; Likewise for emacsclient.rc
-    (set-version-in-file root "nt/emacsclient.rc" comma-version
-			 (rx (and "FILEVERSION" (1+ space)
-				  (submatch (1+ (in "0-9,"))))))
-    (set-version-in-file root "nt/emacsclient.rc" comma-version
-			 (rx (and "PRODUCTVERSION" (1+ space)
-				  (submatch (1+ (in "0-9,"))))))
-    (set-version-in-file root "nt/emacsclient.rc" comma-space-version
-			 (rx (and "\"FileVersion\"" (0+ space) ?, (0+ space)
-				  ?\" (submatch (1+ (in "0-9, "))) "\\0\"")))
-    (set-version-in-file root "nt/emacsclient.rc" comma-space-version
-			 (rx (and "\"ProductVersion\"" (0+ space) ?,
-				  (0+ space) ?\" (submatch (1+ (in "0-9, ")))
-				  "\\0\"")))
-    ;; Major version only.
-    (when (string-match "\\([0-9]\\{2,\\}\\)" version)
-      (setq version (match-string 1 version))
-      (set-version-in-file root "src/msdos.c" version
-			   (rx (and "Vwindow_system_version" (1+ not-newline)
-				    ?\( (submatch (1+ (in "0-9"))) ?\))))
-      (set-version-in-file root "etc/refcards/ru-refcard.tex" version
-			   "\\\\newcommand{\\\\versionemacs}\\[0\\]\
-{\\([0-9]\\{2,\\}\\)}.+%.+version of Emacs")
-      (set-version-in-file root "etc/refcards/emacsver.tex" version
-			   "\\\\def\\\\versionemacs\
+  ;; Major version only.
+  (when (string-match "\\([0-9]\\{2,\\}\\)" version)
+    (let ((newmajor (match-string 1 version)))
+      (set-version-in-file root "src/msdos.c" newmajor
+                           (rx (and "Vwindow_system_version" (1+ not-newline)
+                                    ?\( (submatch (1+ (in "0-9"))) ?\))))
+      (set-version-in-file root "etc/refcards/ru-refcard.tex" newmajor
+                           "\\\\newcommand{\\\\versionemacs}\\[0\\]\
 {\\([0-9]\\{2,\\}\\)}.+%.+version of Emacs")))
+  (let* ((oldversion
+          (with-temp-buffer
+            (insert-file-contents (expand-file-name "README" root))
+            (if (re-search-forward "version \\([0-9.]*\\)" nil t)
+                (version-to-list (match-string 1)))))
+         (oldmajor (if oldversion (car oldversion)))
+         (newversion (version-to-list version))
+         (newmajor (car newversion))
+         (newshort (format "%s.%s" newmajor
+                           (+ (cadr newversion)
+                              (if (eq 2 (length newversion)) 0 1))))
+         (majorbump (and oldversion (not (equal oldmajor newmajor))))
+         (minorbump (and oldversion (not majorbump)
+                         (not (equal (cadr oldversion) (cadr newversion)))))
+         (newsfile (expand-file-name "etc/NEWS" root))
+         (oldnewsfile (expand-file-name (format "etc/NEWS.%s" oldmajor) root)))
+    (when (and majorbump
+               (not (file-exists-p oldnewsfile)))
+      (rename-file newsfile oldnewsfile)
+      (find-file oldnewsfile)           ; to prompt you to commit it
+      (copy-file oldnewsfile newsfile)
+      (with-temp-buffer
+        (insert-file-contents newsfile)
+        (re-search-forward "is about changes in Emacs version \\([0-9]+\\)")
+        (replace-match (number-to-string newmajor) nil nil nil 1)
+        (re-search-forward "^See files \\(NEWS\\)")
+        (replace-match (format "NEWS.%s, NEWS" oldmajor) nil nil nil 1)
+        (let ((start (line-beginning-position)))
+          (search-forward "in older Emacs versions")
+          (or (equal start (line-beginning-position))
+              (fill-region start (line-beginning-position 2))))
+        (re-search-forward "^$")
+        (forward-line -1)
+        (let ((start (point)))
+          (goto-char (point-max))
+          (re-search-backward "^$" nil nil 2)
+          (delete-region start (line-beginning-position 0)))
+        (write-region nil nil newsfile)))
+    (when (or majorbump minorbump)
+      (find-file newsfile)
+      (goto-char (point-min))
+      (if (re-search-forward (format "^\\* .*in Emacs %s" newshort) nil t)
+          (progn
+            (kill-buffer)
+            (message "No need to update etc/NEWS"))
+        (goto-char (point-min))
+        (re-search-forward "^$")
+        (forward-line -1)
+        (dolist (s '("Installation Changes" "Startup Changes" "Changes"
+                     "Editing Changes"
+                     "Changes in Specialized Modes and Packages"
+                          "New Modes and Packages"
+                          "Incompatible Lisp Changes"
+                          "Lisp Changes"))
+          (insert (format "\n\n* %s in Emacs %s\n" s newshort)))
+        (insert (format "\n\n* Changes in Emacs %s on \
+Non-Free Operating Systems\n" newshort)))
+      ;; Because we skip "bump version" commits when merging between branches.
+      ;; Probably doesn't matter in practice, because NEWS changes
+      ;; will only happen on master anyway.
+      (message "Commit any NEWS changes separately")))
   (message "Setting version numbers...done"))
 
 ;; Note this makes some assumptions about form of short copyright.
@@ -189,10 +186,6 @@ Root must be the root of an Emacs source tree."
 		       (rx (and bol "/^#undef " (1+ not-newline)
 				"define COPYRIGHT" (1+ space)
 				?\" (submatch (1+ (not (in ?\")))) ?\")))
-  (set-version-in-file root "nt/config.nt" copyright
-		       (rx (and bol "#" (0+ blank) "define" (1+ blank)
-				"COPYRIGHT" (1+ blank)
-				?\" (submatch (1+ (not (in ?\")))) ?\")))
   (set-version-in-file root "lib-src/rcs2log" copyright
         	       (rx (and "Copyright" (0+ space) ?= (0+ space)
         			?\' (submatch (1+ nonl)))))
@@ -201,7 +194,7 @@ Root must be the root of an Emacs source tree."
     (set-version-in-file root "etc/refcards/ru-refcard.tex" copyright
 			 "\\\\newcommand{\\\\cyear}\\[0\\]\
 {\\([0-9]\\{4\\}\\)}.+%.+copyright year")
-    (set-version-in-file root "etc/refcards/emacsver.tex" copyright
+    (set-version-in-file root "etc/refcards/emacsver.tex.in" copyright
 			 "\\\\def\\\\year\
 {\\([0-9]\\{4\\}\\)}.+%.+copyright year"))
   (message "Setting copyrights...done"))
@@ -252,6 +245,7 @@ Optional argument TYPE is type of output (nil means all)."
 	 (ps-dir (expand-file-name "ps" dest))
 	 (pdf-dir (expand-file-name "pdf" dest))
 	 (emacs (expand-file-name "doc/emacs/emacs.texi" root))
+	 (emacs-xtra (expand-file-name "doc/emacs/emacs-xtra.texi" root))
 	 (elisp (expand-file-name "doc/lispref/elisp.texi" root))
 	 (eintr (expand-file-name "doc/lispintro/emacs-lisp-intro.texi" root))
 	 (misc (manual-misc-manuals root)))
@@ -265,10 +259,14 @@ Optional argument TYPE is type of output (nil means all)."
 	(manual-html-node emacs (expand-file-name "emacs" html-node-dir)))
     (if (member type '(nil "emacs" "emacs-mono"))
 	(manual-html-mono emacs (expand-file-name "emacs.html" html-mono-dir)))
-    (if (member type '(nil "emacs" "emacs-pdf" "pdf"))
-	(manual-pdf emacs (expand-file-name "emacs.pdf" pdf-dir)))
-    (if (member type '(nil "emacs" "emacs-ps" "ps"))
-	(manual-ps emacs (expand-file-name "emacs.ps" ps-dir)))
+    (when (member type '(nil "emacs" "emacs-pdf" "pdf"))
+      (manual-pdf emacs (expand-file-name "emacs.pdf" pdf-dir))
+      ;; emacs-xtra exists only in pdf/ps format.
+      ;; In other formats it is included in the Emacs manual.
+      (manual-pdf emacs-xtra (expand-file-name "emacs-xtra.pdf" pdf-dir)))
+    (when (member type '(nil "emacs" "emacs-ps" "ps"))
+      (manual-ps emacs (expand-file-name "emacs.ps" ps-dir))
+      (manual-ps emacs-xtra (expand-file-name "emacs-xtra.ps" ps-dir)))
     (if (member type '(nil "elisp" "elisp-node"))
 	(manual-html-node elisp (expand-file-name "elisp" html-node-dir)))
     (if (member type '(nil "elisp" "elisp-mono"))
@@ -297,7 +295,7 @@ Optional argument TYPE is type of output (nil means all)."
 
 (defconst manual-meta-string
   "<meta http-equiv=\"content-type\" content=\"text/html; charset=utf-8\">
-<link rev=\"made\" href=\"mailto:webmasters@gnu.org\">
+<link rev=\"made\" href=\"mailto:bug-gnu-emacs@gnu.org\">
 <link rel=\"icon\" type=\"image/png\" href=\"/graphics/gnu-head-mini.png\">
 <meta name=\"ICBM\" content=\"42.256233,-71.006581\">
 <meta name=\"DC.title\" content=\"gnu.org\">\n\n")
@@ -609,6 +607,89 @@ style=\"text-align:left\">")
 	    (setq done t))))
 	(forward-line 1)))))
 
+
+(defconst make-manuals-dist-output-variables
+  `(("@\\(top_\\)?srcdir@" . ".")	; top_srcdir is wrong, but not used
+    ("^\\(\\(?:texinfo\\|buildinfo\\|emacs\\)dir *=\\).*" . "\\1 .")
+    ("^\\(clean:.*\\)" . "\\1 infoclean")
+    ("@MAKEINFO@" . "makeinfo")
+    ("@MKDIR_P@" . "mkdir -p")
+    ("@INFO_EXT@" . ".info")
+    ("@INFO_OPTS@" . "")
+    ("@SHELL@" . "/bin/bash")
+    ("@prefix@" . "/usr/local")
+    ("@datarootdir@" . "${prefix}/share")
+    ("@datadir@" . "${datarootdir}")
+    ("@PACKAGE_TARNAME@" . "emacs")
+    ("@docdir@" . "${datarootdir}/doc/${PACKAGE_TARNAME}")
+    ("@\\(dvi\\|html\\|pdf\\|ps\\)dir@" . "${docdir}")
+    ("@GZIP_PROG@" . "gzip")
+    ("@INSTALL@" . "install -c")
+    ("@INSTALL_DATA@" . "${INSTALL} -m 644")
+    ("@configure_input@" . ""))
+  "Alist of (REGEXP . REPLACEMENT) pairs for `make-manuals-dist'.")
+
+(defun make-manuals-dist--1 (root type)
+  "Subroutine of `make-manuals-dist'."
+  (let* ((dest (expand-file-name "manual" root))
+	 (default-directory (progn (make-directory dest t)
+				   (file-name-as-directory dest)))
+	 (version (with-temp-buffer
+		    (insert-file-contents "../doc/emacs/emacsver.texi")
+		    (re-search-forward "@set EMACSVER \\([0-9.]+\\)")
+		    (match-string 1)))
+	 (stem (format "emacs-%s-%s" (if (equal type "emacs") "manual" type)
+		       version))
+	 (tarfile (format "%s.tar" stem)))
+    (message "Doing %s..." type)
+    (if (file-directory-p stem)
+	(delete-directory stem t))
+    (make-directory stem)
+    (copy-file "../doc/misc/texinfo.tex" stem)
+    (or (equal type "emacs") (copy-file "../doc/emacs/emacsver.texi" stem))
+    (dolist (file (directory-files (format "../doc/%s" type) t))
+      (if (or (string-match-p "\\(\\.texi\\'\\|/README\\'\\)" file)
+	      (and (equal type "lispintro")
+		   (string-match-p "\\.\\(eps\\|pdf\\)\\'" file)))
+	  (copy-file file stem)))
+    (with-temp-buffer
+      (let ((outvars make-manuals-dist-output-variables))
+	(push `("@version@" . ,version) outvars)
+	(insert-file-contents (format "../doc/%s/Makefile.in" type))
+	(dolist (cons outvars)
+	  (while (re-search-forward (car cons) nil t)
+	    (replace-match (cdr cons) t))
+	  (goto-char (point-min))))
+      (let (ats)
+	(while (re-search-forward "@[a-zA-Z_]+@" nil t)
+	  (setq ats t)
+	  (message "Unexpanded: %s" (match-string 0)))
+	(if ats (error "Unexpanded configure variables in Makefile?")))
+      (write-region nil nil (expand-file-name (format "%s/Makefile" stem))
+		    nil 'silent))
+    (call-process "tar" nil nil nil "-cf" tarfile stem)
+    (delete-directory stem t)
+    (message "...created %s" tarfile)))
+
+;; Does anyone actually use these tarfiles?
+(defun make-manuals-dist (root &optional type)
+  "Make the standalone manual source tarfiles for the Emacs webpage.
+ROOT should be the root of an Emacs source tree.
+Interactively with a prefix argument, prompt for TYPE.
+Optional argument TYPE is type of output (nil means all)."
+  (interactive (let ((root (read-directory-name "Emacs root directory: "
+						source-directory nil t)))
+		 (list root
+		       (if current-prefix-arg
+			   (completing-read
+			    "Type: "
+			    '("emacs" "lispref" "lispintro" "misc"))))))
+  (unless (file-exists-p (expand-file-name "src/emacs.c" root))
+    (user-error "%s doesn't seem to be the root of an Emacs source tree" root))
+  (dolist (m '("emacs" "lispref" "lispintro" "misc"))
+    (if (member type (list nil m))
+	(make-manuals-dist--1 root m))))
+
 
 ;; Stuff to check new `defcustom's got :version tags.
 ;; Adapted from check-declare.el.
@@ -654,8 +735,8 @@ If optional argument OLD is non-nil, also scan for `defvar's."
 	      (and (not old)
 		   (equal "custom" (match-string 2))
 		   (not (memq :type form))
-		   (display-warning 'custom
-				    (format "Missing type in: `%s'" form)))
+		   (display-warning
+                    'custom (format-message "Missing type in: `%s'" form)))
 	      (setq ver (car (cdr-safe (memq :version form))))
 	      (if (equal "group" (match-string 2))
 		  ;; Group :version could be old.
@@ -669,7 +750,7 @@ If optional argument OLD is non-nil, also scan for `defvar's."
 		       (setq grp (car (cdr-safe grp))) ; (quote foo) -> foo
 		       (setq ver (assq grp glist))))
 		(setq alist (cons (cons var ver) alist))))
-          (if form (message "Malformed defcustom: `%s'" form)))))
+          (if form (format-message "Malformed defcustom: `%s'" form)))))
     (message "%sdone" m)
     alist))
 
@@ -761,7 +842,8 @@ changes (in a non-trivial way).  This function does not check for that."
 	(message "No missing :version tags")
       (pop-to-buffer "*cusver*")
       (erase-buffer)
-      (insert "These `defcustom's might be missing :version tags:\n\n")
+      (insert (substitute-command-keys
+               "These `defcustom's might be missing :version tags:\n\n"))
       (dolist (elem result)
 	(let* ((str (file-relative-name (car elem) newdir))
 	       (strlen (length str)))
