@@ -1,6 +1,7 @@
 ;;; finder.el --- topic & keyword-based code finder
 
-;; Copyright (C) 1992, 1997-1999, 2001-2014 Free Software Foundation, Inc.
+;; Copyright (C) 1992, 1997-1999, 2001-2016 Free Software Foundation,
+;; Inc.
 
 ;; Author: Eric S. Raymond <esr@snark.thyrsus.com>
 ;; Created: 16 Jun 1992
@@ -52,7 +53,7 @@
     (frames     . "Emacs frames and window systems")
     (games	. "games, jokes and amusements")
     (hardware	. "interfacing with system hardware")
-    (help	. "on-line help systems")
+    (help	. "Emacs help systems")
     (hypermedia . "links between text or other media types")
     (i18n	. "internationalization and character-set support")
     (internal	. "code for Emacs internals, build process, defaults")
@@ -113,10 +114,6 @@ Each element has the form (KEYWORD . DESCRIPTION).")
     st)
   "Syntax table used while in `finder-mode'.")
 
-(defvar finder-font-lock-keywords
-  '(("`\\([^'`]+\\)'" 1 font-lock-constant-face prepend))
-  "Font-lock keywords for Finder mode.")
-
 (defvar finder-headmark nil
   "Internal Finder mode variable, local in Finder buffer.")
 
@@ -196,7 +193,7 @@ from; the default is `load-path'."
 	summary keywords package version entry desc)
     (dolist (d (or dirs load-path))
       (when (file-exists-p (directory-file-name d))
-	(message "Directory %s" d)
+	(message "Scanning %s for finder" d)
 	(setq package-override
 	      (intern-soft
 	       (cdr-safe
@@ -241,7 +238,7 @@ from; the default is `load-path'."
 		    ;; The idea here is that eg calc.el gets to define
 		    ;; the description of the calc package.
 		    ;; This does not work for eg nxml-mode.el.
-		    ((eq base-name package)
+		    ((or (eq base-name package) version)
 		     (setq desc (cdr entry))
 		     (aset desc 0 version)
 		     (aset desc 2 summary)))
