@@ -278,13 +278,13 @@ Should be consistent with the Git config value i18n.logOutputEncoding."
 
 (defun vc-git-mode-line-string (file)
   "Return a string for `vc-mode-line' to put in the mode line for FILE."
-  (let* ((rev (vc-working-revision file))
+  (let* ((rev (vc-working-revision file 'Git))
          (disp-rev (or (vc-git--symbolic-ref file)
                        (substring rev 0 7)))
          (def-ml (vc-default-mode-line-string 'Git file))
          (help-echo (get-text-property 0 'help-echo def-ml))
          (face   (get-text-property 0 'face def-ml)))
-    (propertize (replace-regexp-in-string (concat rev "\\'") disp-rev def-ml t t)
+    (propertize (concat (substring def-ml 0 4) disp-rev)
                 'face face
                 'help-echo (concat help-echo "\nCurrent revision: " rev))))
 
